local _PATH = (...):match('^(.*[%./])[^%.%/]+$') or ''

local road = {}
road.__index = road

local function new(screen, x, y, alpha, data)
	x, y = x or 0, y or 0
	dx, dy = 0, 0
	alpha = alpha or 0
	dAlpha = 0
	data = data or {}
	straight, outCurve, inCurve = {}, {}, {}
	for i = 0, 5 do table.insert(straight, love.graphics.newImage('png/road0'..i..'.png')) end
	for i = 0, 1 do table.insert(outCurve, love.graphics.newImage('png/road1'..i..'.png')) end
	for i = 0, 0 do table.insert(inCurve, love.graphics.newImage('png/road2'..i..'.png')) end
	scale = straight[1]:getWidth() * 2.5
	screen = screen or {}
	return setmetatable({
		x = x, y = y,
		dx = dx, dy = dy,
		alpha = alpha,
		dAlpha = dAlpha,
		data = data,
		straight = straight,
		outCurve = outCurve,
		inCurve = inCurve,
		scale = scale,
		screen = screen
	}, road)
end

function road:draw()
	self.dx, self.dy = self.x, self.y
	self.dAlpha = self.alpha
	setWhite()
	for i = 1, #self.data do
		if type(self.data[i]) == 'number' then
			for j = 1, self.data[i] do self:drawStraight() end
		elseif self.data[i] == 'left' then self:drawLeft()
		elseif self.data[i] == 'right' then self:drawRight()
		elseif self.data[i] == 'start' then self:drawStart()
		elseif self.data[i] == 'end' then self:drawEnd() end
	end
end

function road:drawStraight()
	self.screen:draw(math.randomData(self.straight), self.dx, self.dy, self.dAlpha, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self.screen:draw(math.randomData(self.straight), self.dx + self.scale * math.cos(self.dAlpha), self.dy + self.scale * math.sin(self.dAlpha), self.dAlpha + math.pi, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:goStraight()
end
function road:drawLeft()
	self.screen:draw(math.randomData(self.straight), self.dx, self.dy, self.dAlpha, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self.screen:draw(math.randomData(self.inCurve), self.dx + self.scale * (math.cos(self.dAlpha) + math.sin(self.dAlpha)), self.dy + self.scale * (math.sin(self.dAlpha) - math.cos(self.dAlpha)), self.dAlpha + math.pi / 2, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:goStraight()
	self.screen:draw(math.randomData(self.outCurve), self.dx - self.scale * math.sin(self.dAlpha), self.dy + self.scale * math.cos(self.dAlpha), self.dAlpha - math.pi / 2, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:turnLeft()
	self.screen:draw(math.randomData(self.straight), self.dx, self.dy, self.dAlpha, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())	
	self:goStraight()
end
function road:drawRight()
	self.screen:draw(math.randomData(self.inCurve), self.dx, self.dy, self.dAlpha, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self.screen:draw(math.randomData(self.straight), self.dx + self.scale * math.cos(self.dAlpha), self.dy + self.scale * math.sin(self.dAlpha), self.dAlpha + math.pi, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:goStraight()
	self.screen:draw(math.randomData(self.outCurve), self.dx + self.scale * math.cos(self.dAlpha), self.dy + self.scale * math.sin(self.dAlpha), self.dAlpha + math.pi, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:turnRight()
	self.screen:draw(math.randomData(self.straight), self.dx + self.scale * math.cos(self.dAlpha), self.dy + self.scale * math.sin(self.dAlpha), self.dAlpha + math.pi, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())	
	self:goStraight()
end
function road:drawStart()
	self.screen:draw(math.randomData(self.outCurve), self.dx, self.dy, self.dAlpha, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self.screen:draw(math.randomData(self.outCurve), self.dx + self.scale * (math.cos(self.dAlpha) + math.sin(self.dAlpha)), self.dy + self.scale * (math.sin(self.dAlpha) - math.cos(self.dAlpha)), self.dAlpha + math.pi / 2, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:goStraight()
end
function road:drawEnd()
	self.screen:draw(math.randomData(self.outCurve), self.dx + self.scale * math.cos(self.dAlpha), self.dy + self.scale * math.sin(self.dAlpha), self.dAlpha + math.pi, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self.screen:draw(math.randomData(self.outCurve), self.dx - self.scale * math.sin(self.dAlpha), self.dy + self.scale * math.cos(self.dAlpha), self.dAlpha + 1.5 * math.pi, self.scale / self.straight[1]:getWidth(), self.scale / self.straight[1]:getWidth())
	self:goStraight()
end
function road:goStraight()
	self.dx = self.dx + self.scale * math.cos(self.dAlpha)
	self.dy = self.dy + self.scale * math.sin(self.dAlpha)
end
function road:turnLeft() self.dAlpha = self.dAlpha - math.pi / 2 end
function road:turnRight() self.dAlpha = self.dAlpha + math.pi / 2 end

-- the module
return setmetatable({new = new}, {__call = function(_, ...) return new(...) end})