local _PATH = (...):match('^(.*[%./])[^%.%/]+$') or ''

local screen = {}
screen.__index = screen

local function new(target)
	love.mouse.setVisible(false)
	love.graphics.setBackgroundColor(0, 0, 0, 255)
	love.graphics.setDefaultImageFilter('nearest', 'nearest')
	local font = love.graphics.newImageFont("png/font.png",
    " abcdefghijklmnopqrstuvwxyz" ..
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ0" ..
    "123456789.,!?-+/():;%&`'*#=[]\"")
	love.graphics.setFont(font)
	shader = love.graphics.newPixelEffect([[
		//extern number w, h;
		extern number x, y, z;
		
		vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc){
			vec2 lc = vec2(x, y);
			vec3 lcS = vec3(x, y, z);
			vec3 tcS = vec3(tc.x, tc.y, Texel(tex, tc).r * 0.1);
			number pre = 0.01;
			for (number step = 0; step < 1 + pre; step += pre)
				if ((Texel(tex, tc + (lc - tc) * step).r * 0.1 - tcS.z) > step * (lcS.z - tcS.z))
					return vec4(Texel(tex, tc).rgb * 0.5 * distance(lcS, tcS), Texel(tex, tc).a);
			return vec4(Texel(tex, tc).rgb * distance(lcS, tcS) * distance(lcS, tcS), Texel(tex, tc).a);
		}
	]])
	shader:send('z', 1)
	shadedCanvas = love.graphics.newCanvas()
	target = target or {}
	w = {w = 20, h = 20}
	t = {w = 256, h = 256}
	s = {w = love.graphics.getWidth(), h = love.graphics.getHeight()}
	tiles = {}
	png = {
		hero = love.graphics.newImage('png/hero.png'),
		seed = love.graphics.newImage('png/seed.png'),
	}
	for i = 1, w.w do
		local line = {}
		for j = 1, w.h do
			table.insert(line, love.graphics.newCanvas(t.w, t.h))
		end
		table.insert(tiles, line)
	end
	love.graphics.setBackgroundColor(0, 0, 0, 255)
	love.graphics.setCanvas()
	return setmetatable({
		target = target,
		w = w, t = t, s = s,
		tiles = tiles,
		shader = shader,
		shadedCanvas = shadedCanvas,
		png = png
	}, screen)
end

function screen:drawBackground()
	local x1, x2, y1, y2 = self:getTiles(self:getScreenBox())
	for i = x1, x2 do for j = y1, y2 do
		love.graphics.draw(self.tiles[i][j], (i - 1) * self.t.w, (j - 1) * self.t.h)
	end	end
end

function screen:draw(drawable, x, y, r, sx, sy)
	local x1, x2, y1, y2 = self:getTiles(self:getBoundingBox(drawable, x, y, r, sx, sy))
	for i = x1, x2 do for j = y1, y2 do
		love.graphics.setCanvas(self.tiles[i][j])
		love.graphics.draw(drawable, x - (i - 1) * self.t.w, y - (j - 1) * self.t.h, r, sx, sy)
		love.graphics.setCanvas()
	end	end
end

function screen:getTiles(xMin, xMax, yMin, yMax)
	return math.max(1, math.ceil(xMin / self.t.w)),
		   math.min(math.ceil(xMax / self.t.w), self.w.w),
		   math.max(1, math.ceil(yMin / self.t.h)),
		   math.min(math.ceil(yMax / self.t.h), self.w.h)
end

function screen:getScreenBox()
	return (self.target.x - self.s.w / 2),
		   (self.target.x + self.s.w / 2),
		   (self.target.y - self.s.h / 2),
		   (self.target.y + self.s.h / 2)
end

function screen:getBoundingBox(drawable, x, y, r, sx, sy)
	local W, H = drawable:getWidth() * sx, drawable:getHeight() * sy
	local WH, alpha = math.norm(W, H), math.angle(0, 0, W, H)
	return x + math.min(0, W * math.cos(r), WH * math.cos(r + alpha), H * math.cos(r + math.pi / 2)),
		   x + math.max(0, W * math.cos(r), WH * math.cos(r + alpha), H * math.cos(r + math.pi / 2)),
		   y + math.min(0, W * math.sin(r), WH * math.sin(r + alpha), H * math.sin(r + math.pi / 2)),
		   y + math.max(0, W * math.sin(r), WH * math.sin(r + alpha), H * math.sin(r + math.pi / 2))
end

function screen:attach()
	love.graphics.push()
	love.graphics.translate(self.s.w / 2 - self.target.x, self.s.h / 2 - self.target.y)
	self.shader:send('x', self.target.x / self.s.w)
	self.shader:send('y', (self.s.h - self.target.y) / self.s.h)
end

function screen:detach()
	love.graphics.pop()
end

function screen:setShaderOn()
	love.graphics.setCanvas(self.shadedCanvas)
	love.graphics.clear()
end

function screen:setShaderOff()
	love.graphics.setCanvas()
	love.graphics.setPixelEffect(self.shader)
	love.graphics.draw(self.shadedCanvas)
	love.graphics.setPixelEffect()
end

function screen:getPosition() return self:getX(), self:getY() end
function screen:getX() return self.target.x - self.s.w / 2 + love.mouse.getX() end
function screen:getY() return self.target.y - self.s.h / 2 + love.mouse.getY() end
function screen:getWorldCoords(x, y)
	return self.target.x - self.s.w / 2 + x, self.target.y - self.s.h / 2 + y
end

-- the module
return setmetatable({new = new}, {__call = function(_, ...) return new(...) end})