local _PATH = (...):match('^(.*[%./])[^%.%/]+$') or ''

local seed = {}
seed.__index = seed

local function new(x, y, swarm, screen)
	x, y = x or 0, y or 0
	swarm = swarm or {}
	if math.random() < (2/3) then size = 11
	elseif math.random() < (2/3) then size = 15
	else size = 19 end
	t = math.random()
	alpha = 0
	beta = 0
	distance = 0
	isDead = false
	screen = screen or {}
	png = screen.png.seed or {}
	return setmetatable({
		x = x, y = y,
		swarm = swarm,
		size = size,
		t = t,
		alpha = alpha,
		beta = beta,
		distance = distance,
		isDead = isDead,
		screen = screen,
		png = png
	}, seed)
end

function seed:update(dt)
	self.t = self.t + dt
	if self.t > self.size / 33 then
		self:detach()
		self.t = 0
		if self.swarm.mode == 'expand' then self:move(self.swarm, math.pi)
		elseif self.swarm.mode == 'group' then self:move(self.swarm, 0)
		elseif self.swarm.mode == 'run' then self:move(self.swarm.enemy, 0)
		elseif self.swarm.mode == 'flee' then self:move(self.swarm.enemy, math.pi)
		elseif self.swarm.mode == 'circle' then self:move(self.swarm.enemy, math.pi / 2)
		end
		self:attach()
	end
	self.alpha = math.angle(self.swarm.enemy.x, self.swarm.enemy.y, self.x, self.y)
	self.distance = math.norm(self.swarm.enemy.x - self.x, self.swarm.enemy.y - self.y)
end

function seed:draw()
	love.graphics.setColor(100 + 155 * (19 - self.size) / 19, 0, 0, 255)
	scale = self.size / self.png:getWidth()
	az, er = math.rotate(-self.size / 2, -self.size / 2, self.beta)
	love.graphics.draw(self.png, self.x + az, self.y + er, self.beta, scale, scale)
end

function seed:isHittedBy(weapon)
	if self.size - weapon.damage < 4 then
		table.insert(sms, Message(self.x + math.random(-10, 10), self.y + math.random(-45, -25), 100, 'KILLSHOT', 1))
		weapon.carrier.score = weapon.carrier.score + self.size
		self.isDead = true
		self:detach()
	else
		table.insert(sms, Message(self.x + math.random(-10, 10), self.y + math.random(-45, -25), 100, weapon.damage, 1))
		self:detach()
		self:move(weapon.carrier, math.pi, 5 * weapon.damage * weapon.damage / self.size)
		self.size = self.size - weapon.damage
		self:attach()
	end
end

function seed:move(target, direction, coef)
	self.beta = math.angle(self.x, self.y, target.x, target.y) + direction --[[ALEATOIRE]] + math.pi * 0.125 * (-1 + 2 * math.random()) --[[ALEATOIRE]]
	self.x = self.x + self.size * (coef or 1) * math.cos(self.beta)
	self.y = self.y + self.size * (coef or 1) * math.sin(self.beta)
end

function seed:detach()
	self.swarm.volume = self.swarm.volume - self.size
	self.swarm.x = (self.swarm.x * (self.swarm.volume + self.size) - self.x * self.size) / self.swarm.volume		
	self.swarm.y = (self.swarm.y * (self.swarm.volume + self.size) - self.y * self.size) / self.swarm.volume
end

function seed:attach()
	self.swarm.volume = self.swarm.volume + self.size
	self.swarm.x = (self.swarm.x * (self.swarm.volume - self.size) + self.x * self.size) / self.swarm.volume		
	self.swarm.y = (self.swarm.y * (self.swarm.volume - self.size) + self.y * self.size) / self.swarm.volume
end

-- the module
return setmetatable({new = new}, {__call = function(_, ...) return new(...) end})