local _PATH = (...):match('^(.*[%./])[^%.%/]+$') or ''

local swarm = {}
swarm.__index = swarm

local function new(enemy, distance, screen)
	local theta = 2 * math.pi * math.random()
	x, y = enemy.x + distance * math.cos(theta), enemy.y + distance * math.sin(theta)
	enemy = enemy or {}
	volume = 0
	mode = 'run'
	screen = screen or {}
	return setmetatable({
		x = x, y = y,
		enemy = enemy,
		volume = volume,
		mode = mode,
		screen = screen
	}, swarm)
end

function swarm:addSeeds(quantity)
	local radius = math.sqrt(quantity * 225 / math.pi)
	local xGen, yGen = self.x, self.y
	self.x, self.y = 0, 0
	for i = 1, quantity do
		local r = radius * math.max(0, math.sqrt(math.random()))
		local theta = 2 * math.pi * math.random()
		local seed = Seed(xGen + r * math.cos(theta), yGen + r * math.sin(theta), self, self.screen)
		self.volume = self.volume + seed.size
		self.x = self.x * ((self.volume - seed.size) / self.volume) + seed.x * (seed.size / self.volume)		
		self.y = self.y * ((self.volume - seed.size) / self.volume) + seed.y * (seed.size / self.volume)
		table.insert(self.enemy.enemies, seed)		
	end
end

function swarm:draw()
	love.graphics.setColor(255, 200, 200, 125)
	love.graphics.circle('fill', self.x, self.y, math.max(10, self.volume / 10))
end

-- the module
return setmetatable({new = new}, {__call = function(_, ...) return new(...) end})